//
//	DIB.cpp
//
//	Implementation of CDib class
//


#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include "types.h"
#include "CDib.h"
 

CDib::CDib( UINT nResID )
{
HRSRC			hbmres;

	// Read bits from resource
	m_hBitmap = (HBITMAP)LoadImage( GetModuleHandle(NULL), MAKEINTRESOURCE(nResID), IMAGE_BITMAP, 0,0, LR_CREATEDIBSECTION );

	// Get palette
	m_pal_loaded = FALSE;
	m_hPalette = 0 ;

	hbmres = FindResource( NULL, MAKEINTRESOURCE(nResID), RT_BITMAP );

	if( hbmres ) {

		LPBITMAPINFOHEADER	lpbminfo = (LPBITMAPINFOHEADER)LockResource( LoadResource(NULL,hbmres) );
		int					num_entries;
		HANDLE				hlogpal;
		LPLOGPALETTE		lplogpal;


		if( lpbminfo && (lpbminfo->biSize >= sizeof(BITMAPINFOHEADER) ) && (lpbminfo->biBitCount <= 8) ) {

			RGBQUAD		*entries = (RGBQUAD*)((BYTE*)lpbminfo + lpbminfo->biSize);
			if( lpbminfo->biClrUsed == 0 )
				num_entries = 1 << lpbminfo->biBitCount;
			else
				num_entries = lpbminfo->biClrUsed;

			m_Width = lpbminfo->biWidth;
			m_Height = lpbminfo->biHeight;

			m_InfoSize = sizeof(BITMAPINFO) + sizeof(RGBQUAD)*num_entries;

			m_hInfo = GlobalAlloc( GHND, m_InfoSize );

			if( !m_hInfo )
				return;

			m_pInfo = (LPBITMAPINFO)GlobalLock( m_hInfo );

			if( !m_pInfo )
				return;


			// Copy BITMAPINFO
			memcpy( m_pInfo, lpbminfo, m_InfoSize );

			// Allocate storage for LOGPALETTE
			hlogpal = GlobalAlloc( GHND, sizeof(LOGPALETTE) + sizeof(PALETTEENTRY)*num_entries );

			if( hlogpal ) {

				lplogpal = (LPLOGPALETTE)GlobalLock(hlogpal);

				lplogpal->palVersion = 0x300;
				lplogpal->palNumEntries = num_entries;

				// Copy entries into LOGPALETTE
				for( int i=0; i < num_entries; i++ ) {

					lplogpal->palPalEntry[i].peBlue		= entries[i].rgbBlue;
					lplogpal->palPalEntry[i].peGreen	= entries[i].rgbGreen;
					lplogpal->palPalEntry[i].peRed		= entries[i].rgbRed;
					lplogpal->palPalEntry[i].peFlags	= 0;
				}

				// Create the palette
				m_hPalette = CreatePalette( lplogpal );

				GlobalUnlock( hlogpal );
				GlobalFree( hlogpal );

				m_pal_loaded = TRUE;
			}

		}


	}

}




CDib::CDib( HDC hDC, CDib * based_on )
{
WORD		*pEntry;
int			i;
HANDLE		hLogPal;

	m_Width = based_on->m_Width;
	m_Height = based_on->m_Height;

	m_InfoSize = sizeof(BITMAPINFOHEADER) + (sizeof(WORD) * 256);

	if( !m_hInfo )
		return;

	m_hInfo = GlobalAlloc( GHND, m_InfoSize );

	if( !m_hInfo )
		return;

	m_pInfo = (LPBITMAPINFO)GlobalLock( m_hInfo );

	if( !m_pInfo )
		return;

	// Copy all the header shtuf
	memcpy( m_pInfo, based_on->m_pInfo, sizeof(BITMAPINFOHEADER) );


	// Build coloUr table
	m_pInfo->bmiHeader.biClrUsed = 0;

	pEntry = (WORD*)((BYTE*)m_pInfo + sizeof(BITMAPINFOHEADER) );

	for( i=0; i<256; i++ )
		*pEntry++ = (WORD)i;

	hLogPal = GlobalAlloc( GHND, sizeof(LOGPALETTE) + sizeof(PALETTEENTRY)*256 );


	if( hLogPal ) {

		// Grab a copy of the DC's palette
		HPALETTE		hTempPal = GetCurrentObject(hDC,OBJ_PAL);
		LOGPALETTE		*pLogPal = (LOGPALETTE*)GlobalLock( hLogPal );

		pLogPal->palVersion = 0x300;
		pLogPal->palNumEntries = 256;

		// Copy pal from DC
		GetPaletteEntries( hTempPal, 0,256, pLogPal->palPalEntry );

		// Create the new pal
		m_hPalette = CreatePalette( pLogPal );

		// Clean up
		GlobalUnlock( hLogPal );
		GlobalFree( hLogPal );
	}

	// Create and attach the DIB section
	m_hBitmap = CreateDIBSection( hDC, m_pInfo, DIB_RGB_COLORS, &m_pBits, NULL, 0 );

}




CDib::CDib( HDC hDC, int width, int height )
{
WORD		*pEntry;
int			i;
HANDLE		hLogPal;


	m_Width = width;
	m_Height = height;

	m_InfoSize = sizeof(BITMAPINFOHEADER) + (sizeof(WORD) * 256);

	if( !m_hInfo )
		return;

	m_hInfo = GlobalAlloc( GHND, m_InfoSize );

	if( !m_hInfo )
		return;

	m_pInfo = (LPBITMAPINFO)GlobalLock( m_hInfo );

	if( !m_pInfo )
		return;

//	// Copy all the header shtuf
//	memcpy( m_pInfo, based_on->m_pInfo, sizeof(BITMAPINFOHEADER) );
//bitmapinfo
	m_pInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	m_pInfo->bmiHeader.biWidth = width;
	m_pInfo->bmiHeader.biHeight = height;
	m_pInfo->bmiHeader.biPlanes = 1;
	m_pInfo->bmiHeader.biBitCount = 8;
	m_pInfo->bmiHeader.biSizeImage = 0;
	m_pInfo->bmiHeader.biClrUsed = 0;
	m_pInfo->bmiHeader.biClrImportant = 0;
//bitmapinfo


	// Build coloUr table
	pEntry = (WORD*)((BYTE*)m_pInfo + sizeof(BITMAPINFOHEADER) );

	for( i=0; i<256; i++ )
		*pEntry++ = (WORD)i;

	hLogPal = GlobalAlloc( GHND, sizeof(LOGPALETTE) + sizeof(PALETTEENTRY)*256 );


	if( hLogPal ) {

		// Grab a copy of the DC's palette
		HPALETTE	hTempPal = GetCurrentObject(hDC,OBJ_PAL);
		LOGPALETTE	*pLogPal = (LOGPALETTE*)GlobalLock( hLogPal );

		pLogPal->palVersion = 0x300;
		pLogPal->palNumEntries = 256;

		// Copy pal from DC
		GetPaletteEntries( hTempPal, 0,256, pLogPal->palPalEntry );

		// Create the new pal
		m_hPalette = CreatePalette( pLogPal );

		// Clean up
		GlobalUnlock( hLogPal );
		GlobalFree( hLogPal );
	}

	// Create and attach the DIB section
	m_hBitmap = CreateDIBSection( hDC, m_pInfo, DIB_RGB_COLORS, &m_pBits, NULL, 0 );

}




CDib::CDib( HDC hDC, int width, int height, HPALETTE hPalette )
{
RGBQUAD			*pEntry;
int				i;
HANDLE			hLogPal;
LOGPALETTE		*pLogPal;
PALETTEENTRY	*pPalList;


	m_Width = width;
	m_Height = height;

	m_InfoSize = sizeof(BITMAPINFOHEADER) + (sizeof(PALETTEENTRY) * 256);

	if( !m_hInfo )
		return;

	m_hInfo = GlobalAlloc( GHND, m_InfoSize );

	if( !m_hInfo )
		return;

	m_pInfo = (LPBITMAPINFO)GlobalLock( m_hInfo );

	if( !m_pInfo )
		return;

//	// Copy all the header shtuf
//	memcpy( m_pInfo, based_on->m_pInfo, sizeof(BITMAPINFOHEADER) );
//bitmapinfo
	m_pInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	m_pInfo->bmiHeader.biWidth = width;
	m_pInfo->bmiHeader.biHeight = -height;
	m_pInfo->bmiHeader.biPlanes = 1;
	m_pInfo->bmiHeader.biBitCount = 8;
	m_pInfo->bmiHeader.biSizeImage = 0;
	m_pInfo->bmiHeader.biClrUsed = 256;
	m_pInfo->bmiHeader.biClrImportant = 256;
//bitmapinfo


	// Build coloUr table
	hLogPal = GlobalAlloc( GHND, sizeof(LOGPALETTE) + sizeof(PALETTEENTRY)*256 );
	pLogPal = (LOGPALETTE*)GlobalLock( hLogPal );

	pLogPal->palVersion = 0x300;
	pLogPal->palNumEntries = 256;
	pPalList = pLogPal->palPalEntry;

	GetPaletteEntries( hPalette, 0,256, pLogPal->palPalEntry );
	m_hPalette = CreatePalette( pLogPal );

	pEntry = (RGBQUAD*)((BYTE*)m_pInfo + sizeof(BITMAPINFOHEADER) );

	// copy coloUrs into DIBsection
	for( i=0; i<256; i++, pEntry++, pPalList++ ) {
		pEntry->rgbRed = pPalList->peRed;
		pEntry->rgbGreen = pPalList->peGreen;
		pEntry->rgbBlue = pPalList->peBlue;
		pEntry->rgbReserved = 0;
	}

		// Clean up
	GlobalUnlock( hLogPal );
	GlobalFree( hLogPal );

	// Create and attach the DIB section
	m_hBitmap = CreateDIBSection( hDC, m_pInfo, DIB_RGB_COLORS, &m_pBits, NULL, 0 );

}




CDib::~CDib()
{
	DeleteObject ( m_hPalette );
}




void CDib::Draw( HDC hDC, int dx, int dy, int sx, int sy, int sw, int sh )
{
HDC			dcMem;
HPALETTE	OldPal;
HBITMAP		OldBmp;

	OldPal = SelectPalette( hDC, m_hPalette, FALSE );
	RealizePalette( hDC );

	dcMem = CreateCompatibleDC( hDC );

	OldBmp = SelectObject( dcMem, m_hBitmap );

	// Blit it on!
	BitBlt( hDC, dx,dy, sw,sh, dcMem, sx,sy, SRCCOPY );

	// Tidy up
	SelectObject( dcMem, OldBmp );
	
	SelectPalette( hDC, OldPal, FALSE );		//wont this destroy
	RealizePalette( hDC );						//the current colours?

}

void CDib::Draw( HDC hDC, int dx, int dy, int sx, int sy, int sw, int sh, int scale )
{
HDC			dcMem;
HPALETTE	OldPal;
HBITMAP		OldBmp;

	OldPal = SelectPalette( hDC, m_hPalette, FALSE );
	RealizePalette( hDC );

	dcMem = CreateCompatibleDC( hDC );

	OldBmp = SelectObject( dcMem, m_hBitmap );

	// Blit it on!
	StretchBlt( hDC, dx*scale,dy*scale, sw*scale,sh*scale, dcMem, sx,sy, sw, sh, SRCCOPY );

	// Tidy up
	SelectObject( dcMem, OldBmp );
	
	SelectPalette( hDC, OldPal, FALSE );		//wont this destroy
	RealizePalette( hDC );						//the current colours?

}




void CDib::Save ( char* filename )
{
FILE*		bmpfile;

BITMAPFILEHEADER bfh;
BITMAPINFOHEADER bih;

HANDLE			hLogPal;
LOGPALETTE		*pLogPal;

int			ii;
int			size;

	CLEARSTRUCT ( bfh );
	CLEARSTRUCT ( bih );

	size = m_Width*m_Height;

	bfh.bfType = 'MB';
	bfh.bfSize = sizeof ( bfh ) +sizeof( bih ) + size;
	bfh.bfOffBits = sizeof ( bfh ) +sizeof( bih );

	bih.biSize = sizeof ( bih );
	bih.biWidth = m_Width ;
	bih.biHeight = m_Height ;
	bih.biPlanes = 1;
	bih.biBitCount = 8;
	bih.biCompression = BI_RGB;
	bih.biSizeImage = size ;
	bih.biClrUsed = 256;
	bih.biClrImportant = bih.biClrUsed;

	// Build coloUr table
	int palsize = sizeof(LOGPALETTE) + sizeof(PALETTEENTRY)*bih.biClrUsed;
	hLogPal = GlobalAlloc( GHND, palsize );
	pLogPal = (LOGPALETTE*)GlobalLock( hLogPal );

	GetPaletteEntries( m_hPalette, 0,bih.biClrUsed, pLogPal->palPalEntry );


	if ( (bmpfile = fopen( filename, "wb+" )) != NULL )
	{
		/* Write the BMP header */
		fwrite ( &bfh, sizeof( bfh ), 1, bmpfile );
		fwrite ( &bih, sizeof( bih ), 1, bmpfile );

		for ( ii=0; ii<bih.biClrUsed; ii++ ) {
			//fwrite ( pLogPal->palPalEntry, palsize, 1, bmpfile );
			fwrite ( pLogPal->palPalEntry, palsize, 1, bmpfile );
		}

		/* Write the image data BACKWARDS! */
		u8* image = (u8*)m_pBits + ( (m_Height-1) * m_Width );
		for (ii=0; ii<m_Height; ii++) {
			fwrite ( image, m_Width, 1, bmpfile );
			image-= m_Width;
		}

		fclose ( bmpfile );

	}

	// Clean up
	GlobalUnlock( hLogPal );
	GlobalFree( hLogPal );
	



}
