#include "types.h"
#include "resource.h"
#include "nodes.h"

#define ON_MSG(message, fn)    \
    case (message): return HANDLE_##message((hWnd), (wParam), (lParam), (fn))


int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow);
BOOL InitApplication ( HINSTANCE hInstance );
LRESULT CALLBACK WndProc ( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );
LRESULT CALLBACK AboutWndProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
static void OnPaletteChanged(HWND hwnd, HWND hwndPaletteChange);
static BOOL OnQueryNewPalette(HWND hWnd);
static void OnDestroy(HWND hwnd);
static void OnPaint(HWND hwnd);
static void OnRButtonDown(HWND hWnd, BOOL fDoubleClick, int x, int y, UINT keyFlags);


extern char szAppName[];
extern char szTitle[];
extern HPALETTE	hPalSpec;


HINSTANCE	hInst;
HWND		hWndMain;


int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
MSG		msg;
HANDLE	hAccelTable;

	if (!hPrevInstance)
		if (!InitApplication(hInstance))
			return (FALSE);

	if (!InitInstance(hInstance, nCmdShow))
		return (FALSE);

	nodes_Init();

	ShowWindow(hWndMain, nCmdShow);
	UpdateWindow(hWndMain);

	hAccelTable = LoadAccelerators (hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR1));
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	nodes_DeInit();

	return (msg.wParam);
	
}



BOOL InitApplication ( HINSTANCE hInstance )
{
WNDCLASS  wc;

	// Fill in window class structure with parameters that describe
	// the main window.
	wc.style         = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc   = (WNDPROC)WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon         = LoadIcon (hInstance, szAppName);
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU1);
	wc.lpszClassName = szAppName;


	return RegisterClass(&wc);

}


BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
RECT rect;

	hInst = hInstance; // Store instance handle in our global variable

	int style=WS_OVERLAPPED|WS_MINIMIZEBOX|WS_CAPTION|WS_SYSMENU;

	SetRect ( &rect, 0, 0, SCREEN_WIDTH*2, SCREEN_HEIGHT*2 );

	AdjustWindowRectEx( &rect, style, FALSE, 0 );

	hWndMain = CreateWindow(szAppName, szTitle, style,
		CW_USEDEFAULT, 0, (rect.right-rect.left),(rect.bottom-rect.top),
		NULL, NULL, hInstance, NULL);

	if (!hWndMain)
		return (FALSE);

	return (TRUE);
}

BEGIN_MESSAGE_MAP( WndProc )
		ON_MSG ( WM_COMMAND, nodes_OnCommand );
		ON_MSG ( WM_PAINT, OnPaint );
		ON_MSG ( WM_DESTROY, OnDestroy );
		ON_MSG ( WM_QUERYNEWPALETTE, OnQueryNewPalette );
		ON_MSG ( WM_PALETTECHANGED, OnPaletteChanged );
END_MESSAGE_MAP	


static void OnPaint(HWND hWnd)
{
PAINTSTRUCT ps;
HDC hdc;

	hdc = BeginPaint (hWnd, &ps);
	nodes_Paint(hdc);
	EndPaint (hWnd, &ps);
}

static void OnDestroy(HWND hWnd)
{
	// Tell WinHelp we don't need it any more...
	WinHelp (hWnd, APPNAME".HLP", HELP_QUIT,(DWORD)0);
	PostQuitMessage(0);

}


static void OnPaletteChanged(HWND hWnd, HWND hwndPaletteChange)
{
HDC hDC;           // Handle to device context
HPALETTE hOldPal;  // Handle to previous logical palette
 
	// If this application did not change the palette, select
	// and realize this application's palette
	if (hwndPaletteChange != hWnd)
	{

	   // Need the window's DC for SelectPalette/RealizePalette
	   hDC = GetDC(hWnd);

	   // Select and realize hPalette
	   hOldPal = SelectPalette(hDC, hPalSpec, FALSE);
	   RealizePalette(hDC);

	   // When updating the colors for an inactive window,
	   // UpdateColors can be called because it is faster than
	   // redrawing the client area (even though the results are
	   // not as good)
	   UpdateColors(hDC);

	   // Clean up
	   if (hOldPal)
		  SelectPalette(hDC, hOldPal, FALSE);
	   ReleaseDC(hWnd, hDC);

	}
}
 
/*
NOTE: The WM_PALETTECHANGED message is sent to all top-level and
overlapped windows; therefore, if any child window uses a color
palette, this message must be passed on to it.
 
The WM_QUERYNEWPALETTE message informs a window that it is about to
receive input focus. In response, the window receiving focus should
realize its palette as a foreground palette and update its client
area. If the window realizes its palette, it should return TRUE;
otherwise, it should return FALSE. The following sample code
demonstrates processing WM_QUERYNEWPALETTE:
*/


static BOOL OnQueryNewPalette(HWND hWnd)
{
HDC hDC;           // Handle to device context
HPALETTE hOldPal;  // Handle to previous logical palette


 
    // Need the window's DC for SelectPalette/RealizePalette
    hDC = GetDC(hWnd);

    // Select and realize hPalette
    hOldPal = SelectPalette(hDC, hPalSpec, FALSE);
    RealizePalette(hDC);

    // Redraw the entire client area
    InvalidateRect(hWnd, NULL, TRUE);
    UpdateWindow(hWnd);

    // Clean up
    if (hOldPal)
       SelectPalette(hDC, hOldPal, FALSE);

    ReleaseDC(hWnd, hDC);

    // Message processed, return TRUE

    return TRUE;

}
